var ChecksCompendium = OpenTextfile('https://dl.dropboxusercontent.com/u/9207945/CompGen/Output/Compendiums_txt/OU_Checks_Sorted.txt').join('\n');
var FullEntry = [];
{
	var Entries = ChecksCompendium.split('\n\nNewline\nNewline\n\n');
	for (var i in Entries) {
		FullEntry[i] = ParseCompendium(Entries[i], true);
	}
	ChecksCompendium = Entries.join('\n\n').split('\n');
}
var ThreatNum = -2;

// the temporary place to collect what we're parsing
var filtered = [];

//
// 3d array with string lists in it
// usage: Threats[1][2][3] will hold the information on all threats
// that you have one GSI, two SSI and three NSI Checks for
//
// that's because we are doing heap sort.
// when assembling the output, those lists will be appended to each other
// starting with the 0GSI 0SSI 0NSI one, then 0 0 1, 0 0 2, etc.
//
var Threats = [];
for (var cGSI = 0; cGSI < 45; cGSI++) {
	Threats[cGSI] = [];
	for (var cSSI = 0; cSSI < 45; cSSI++) {
		Threats[cGSI][cSSI] = [];
		for (var cNSI = 0; cNSI < 45; cNSI++) {
			Threats[cGSI][cSSI][cNSI] = [];
		}
	}
}


// check modes (the current heading under which we're parsing)
const cmGSI = 0;         // heading "GSI:"
const cmSSI = 1;         // heading "SSI:"
const cmNSI = 2;         // heading "NSI:"
const cmNextThreat = 3;  // heading " Checks:", for example "Tyranitar Checks:"

const Headings = ['GSI', 'SSI', 'NSI', ' Checks', 'Newline'];

var Count = [];  // Track the number of GSIs, SSIs, NSIs to this threat
Count[cmGSI] = 0;
Count[cmSSI] = 0;
Count[cmNSI] = 0;

function CreateThreatlist(team) {
	// team must be a comma-separated list of dex numbers or names
	if (team.length == 0)
		team = ['|']
	else {
		team = team.split(',');
		for (i = 0; i < team.length; i++)
			team[i] = AutocompletePokemon(team[i]);
	}
	
	filtered = [];
	
	Threats = [];
	for (var cGSI = 0; cGSI < 45; cGSI++) {
		Threats[cGSI] = [];
		for (var cSSI = 0; cSSI < 45; cSSI++) {
			Threats[cGSI][cSSI] = [];
			for (var cNSI = 0; cNSI < 45; cNSI++) {
				Threats[cGSI][cSSI][cNSI] = [];
			}
		}
	}
	ThreatNum = -2;
	
	// the list of keywords that we're looking for inside the Checks Compendium
	var Watchlist = ['GSI', 'SSI', 'NSI', ' Checks', 'Newline'];
	Watchlist.push.apply(Watchlist, team);  // append the second array to the first
	
	//
	// "watchlist" should look like this now (for example)
	// GSI
	// SSI
	// NSI
	//  Checks:
	// Newline
	// 305-m        | Garchomp-Mega     |
	// 376          | Metagross         |
	// 530          | Excadrill         |
	// 630          | Mandibuzz         |
	// 490          | Manaphy           |
	// 300          | Hippowdon         |
	//
	
	// check mode (the current heading under which we're parsing)
	var Mode = cmNextThreat;
	Count = [0, 0, 0];  // Number of GSIs, SSIs, NSIs to this threat
	
	// parse the compendium..
	
	// loop through ChecksCompendium and Watchlist
	var line = 0;  // line in ChecksCompendium
	var w = 0;     // line in Watchlist
	while (line < ChecksCompendium.length) {
		if (  ChecksCompendium[line].indexOf(Watchlist[w]) >= 0  ) {
			// found a keyword
			if (w <= 2)
				Mode = w        // keyword was: the beginning of a new check mode (GSI->SSI->NSI)
			else if (w == 3) {
				ClearThreat();  // keyword was: the beginning of next threat. save "filtered", and clear it
			} else if (w >= 5)
				Count[Mode]++;  // keyword was: a check that "team" has
			
			// collect any lines that have a keyword, no matter what it is
			filtered.push(ChecksCompendium[line]);
			
			// already added the current line, so stop looking for keywords and start with the next
			w = Watchlist.length;
		}
		
		w++;
		if (w >= Watchlist.length) {
			line++;
			w = 0;
			// keep empty lines (just for formating the source, this won't be visible to the user)
			while (  (line < ChecksCompendium.length) && (ChecksCompendium[line].trim().length == 0)  ) {
				filtered.push("");
				line++;
			}
		}
	}
	
	
	// finished the final threat
	// it's still floating in filtered, clear one last time to save it
	ClearThreat();
	
	// now add everything together to create the output
	var output = [];
	for (var j = 0; j < team.length; j++) {
		if (team[j] == "|")
			team[j] = ""
		else team[j] = MakeClickable(team[j]);
	}
	output.push.apply(output, team);
	output.push("", "<h3>Threatlist</h3>", "");
	for (var gsiCount = 0; gsiCount < Threats.length; gsiCount++) {
		for (var ssiCount = 0; ssiCount < Threats[gsiCount].length; ssiCount++) {
			for (var nsiCount = 0; nsiCount < Threats[gsiCount][ssiCount].length; nsiCount++) {
				if (Threats[gsiCount][ssiCount][nsiCount].length > 0) {
					output.push("(GSI/SSI/NSI: " + gsiCount + " / " + ssiCount + " / " + nsiCount + ")",
					          "", "Newline", "Newline", "");
					output.push.apply(output, Threats[gsiCount][ssiCount][nsiCount]);
				}
			}
		}
	}
	
	Threats = [];  // let the garbage collection take care of this huge object (saved in a global variable)
	
	return output.join("\n");
}

// We call this whenever we finish parsing a Threat in the Checks Compendium,
// to clean up the filtered variable and counters.
function ClearThreat() {
	if (++ThreatNum == -1)
		return;
	
	// delete any headings ('GSI:', 'SSI:', 'NSI:') without entries
	var lastHeading = -1;
	for (var h = cmGSI; h <= cmNSI; h++) {
		for (var t = 0; t < filtered.length; t++) {
			if (filtered[t] == undefined)
				continue;
			if (filtered[t].indexOf(Headings[h]) >= 0) {
				if (Count[h] > 0)
					lastHeading = t
				else if (t != lastHeading) {
					delete filtered[t];
					if (  (t > 0) && (filtered[t-1].trim() == 'Newline')  ) {
						// also delete proceeding Newline
						// to completely remove the now empty line
						delete filtered[t-1];
					}
				break;
				}
			}
		}
	}
	
	// save this threat's checks and reset everything for the next
//	if (Count[cmGSI] >= Threats.length)
//		Threats[Count[cmGSI]] = [];
//	if (Count[cmSSI] >= Threats[Count[cmGSI]].length)
//		Threats[Count[cmGSI]][Count[cmSSI]] = [];
//	if (Count[cmNSI] >= Threats[Count[cmGSI]][Count[cmSSI]].length)
//		Threats[Count[cmGSI]][Count[cmSSI]][Count[cmNSI]] = [];
//	if (Threats[Count[cmGSI]][Count[cmSSI]][Count[cmSSI]] == undefined)
//		Threats[Count[cmGSI]][Count[cmSSI]][Count[cmSSI]] = [];
	var temp = '<p><div id="cont' + ThreatNum + '" style="display:inline-block" '
	     +         'onmouseenter="OnFirstMouseEnter(' + ThreatNum + ');" '
	     +         'onmouseleave="this.childNodes[1].style.display = \'block\'; this.childNodes[3].style.display = \'none\';">\n'
	     +   '<div style="display:block">\n' + filtered.join('\n') + '\n</div>\n'
	     +   '<div style="display:none"></div>\n'
	     + '</div></p>';
	temp = temp.split('\n');
	Array.prototype.push.apply( Threats[Count[cmGSI]][Count[cmSSI]][Count[cmNSI]], temp );
	filtered = [];
	Count[cmGSI] = 0;
	Count[cmSSI] = 0;
	Count[cmNSI] = 0;
}

function OnFirstMouseEnter(ThreatNumber) {
	var Container = document.getElementById('cont' + ThreatNumber);
	Container.childNodes[3].innerHTML = FullEntry[ThreatNumber];
	Container.childNodes[1].style.display = 'none';
	Container.childNodes[3].style.display = 'block';
//	Container.onmouseenter = 'function() { this.childNodes[1].style.display = \'none\'; this.childNodes[3].style.display = \'block\'; };';
//	Container.onmouseenter();
}